/* Implementation of TLBase64Stream classes.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLBase64Stream.m,v 1.4 1996/01/23 12:18:28 tiggr Exp $  */

#define TLSTREAMSTREAM_DECLARE_PRIVATE_METHODS
#import "tl/TLBase64Stream.h"
#import "tl/TLPatchedRoots.h"
#import "tl/TLString.h"

#define PAD	-3
#define SKIP	-2
#define WARN	-1

static signed char input_encoding[256] =
{
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -2, -1, -1, -2, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  /* ` ' */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
  /* `0' */
  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1,PAD, -1, -1,
  /* `@' */
  -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
  /* `P' */
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
  /* `\`' */
  -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
  /* `p' */
  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,

  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
};

#define PAD_CHAR '='

static char output_encoding[64] =
{
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
  'w', 'x', 'y', 'z', '0', '1', '2', '3',
  '4', '5', '6', '7', '8', '9', '+', '/', 
};

@implementation TLBase64Stream

@end

@implementation TLBase64InputStream

/******************** public methods ********************/

+(TLBase64InputStream *) inputStreamWithStream:
  (id <TLStream, TLInputStream>) str
{
  return ([[self gcAlloc] initWithStream: str]);
} /* +inputStreamWithStream: */

/******************** TLStream ********************/

-close
{
  return (self);
} /* -close */

/******************** TLInputStream ********************/

-(int) readByte
{
  unsigned char c;
  int r = [self readBytes: 1 intoBuffer: &c];
  return (r == 1 ? c : TL_EOF);
} /* -readByte */

-(int) readBytes: (int) n intoBuffer: (char *) b
{
  int d, done = 0;

  while (done != n && !eof)
    {
      d = [stream readByte];
      if (d == TL_EOF)
	eof = 1;
      else
	{
	  int c = input_encoding[d];
	  if (c >= 0)
	    {
	      buffer = (buffer << 6) + c;
	      bits += 6;
	      if (bits >= 8)
		{
		  bits -= 8;
		  b[done++] = buffer >> bits;
		}
	    }
	  else if (c == PAD)
	    {
	      bits = (bits + 6) % 8;
	      if (!bits)
		eof = 1;
	    }
	  else if (c == WARN && !warned)
	    {
	      warned = 1;
	      [self warning: "possibly bad Base64 input `%c'", d];
	    }
	}
    }
  return (done);
} /* -readBytes:intoBuffer: */

@end

@implementation TLBase64OutputStream

+(TLBase64OutputStream *) outputStreamWithStream:
  (id <TLStream, TLOutputStream>) str
{
  return ([[self gcAlloc] initWithStream: str]);
} /* +outputStreamWithStream: */

/******************** TLStream ********************/

-close
{
  id r = self;

  if (bits)
    {
      buffer &= ((1 << bits) - 1);
      bits -= 2;
      if ([stream writeByte: output_encoding[buffer]] == TL_EOF)
	r = nil;
      else
	{
	  while (bits)
	    {
	      if ([stream writeByte: PAD_CHAR] == TL_EOF)
		{
		  r = nil;
		  break;
		}
	      bits -= 2;
	    }
	  if (r && [stream writeByte: '\n'] == TL_EOF)
	    r = nil;
	}
    }
  eof = 1;
  return (r);
} /* -close */

-(int) fileDescriptor
{
  return (stream ? [stream fileDescriptor] : -1);
} /* -fileDescriptor */

-streamp
{
  return (self);
} /* -streamp */

/******************** TLOutputStream ********************/

-flushOutput
{
  /* We don't do no flush.  */
  return (self);
} /* -flushOutput */

-(int) writeByte: (char) c
{
  int r = [self writeBytes: 1 fromBuffer: &c];
  return (r == 1 ? c : TL_EOF);
} /* -writeByte: */

-(int) writeBytes: (int) n fromBuffer: (const char *) b
{
  int done = 0;

  while (done != n && !eof)
    {
      unsigned char c = b[done++];

      buffer = buffer << 8 | c;
      bits += 8;

      while (bits >= 6)
	{
	  int e = (buffer >> (bits - 6)) & ((1 << 6) - 1);
	  bits -= 6;
	  if ([stream writeByte: output_encoding[e]] == TL_EOF)
	    {
	      eof = 1;
	      break;
	    }
	  /* This number is equal to what PINE produces.  */
	  if (++col == 60)
	    {
	      col = 0;
	      if ([stream writeByte: '\n'] == TL_EOF)
		{
		  eof = 1;
		  break;
		}
	    }
	}
    }
  return (done);
} /* -writeBytes:fromBuffer: */

@end
