/* Implementation of TLSneakStream class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLSneakStream.m,v 1.2 1996/01/23 12:18:39 tiggr Exp $  */

#define TLSTREAMSTREAM_DECLARE_PRIVATE_METHODS
#import "tl/TLSneakStream.h"
#import "tl/support.h"

@implementation TLSneakStream

/******************** TLStream ********************/

-close
{
  id r = [stream close];
  formac (V_stderr_, @"<<[%# close] -> %#>>\n", stream, r);
  return (r);
} /* -close */

-(int) fileDescriptor
{
  int r = [stream fileDescriptor];
  formac (V_stderr_, @"<<[%# fileDescriptor] -> %d>>\n", stream, r);
  return (r);
} /* -fileDescriptor */

-streamp
{
  id r = [stream streamp];
  formac (V_stderr_, @"<<[%# streamp] -> %#>>\n", stream, r);
  return (r);
} /* -streamp */

/******************** TLInputStream ********************/

-flushInput
{
  id r = [stream flushInput];
  formac (V_stderr_, @"<<[%# flushInput] -> %#>>\n", stream, r);
  return (r);
} /* -flushInput */

-(int) readByte
{
  int i = [stream readByte];
  formac (V_stderr_, @"<<[%# readByte] -> %d>>\n", stream, i);
  return (i);
} /* -readByte */

-(int) readBytes: (int) n intoBuffer: (char *) b
{
  int i = [stream readBytes: n intoBuffer: b];

  formac (V_stderr_, @"<<[%# readBytes: %d] -> %d %#>>\n", stream, n, i,
	  i > 0 ? [TLString stringWithCString: b length: i] : @"");

  return (i);
} /* -readBytes:intoBuffer: */

/******************** TLOutputStream ********************/

-flushOutput
{
  id r = [stream flushOutput];
  formac (V_stderr_, @"<<[%# flushOutput] -> %#>>\n", stream, r);
  return (r);
} /* -flushOutput */

-(int) writeByte: (char) c
{
  int i = [stream writeByte: c];
  formac (V_stderr_, @"<<[%# writeByte: %c (%d)] -> %d>>\n", stream, c, c, i);
  return (i);
} /* -writeByte: */

-(int) writeBytes: (int) n fromBuffer: (const char *) b
{
  int i = [stream writeBytes: n fromBuffer: b];

  formac (V_stderr_, @"<<[%# writeBytes: %d (%#)] -> %d>>\n", stream, n,
	  i > 0 ? [TLString stringWithCString: b length: n] : @"", i);

  return (i);
} /* -writeBytes:fromBuffer: */

@end
