/* Implementation of TLSocketStream classes.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLSocketStream.m,v 1.8 1996/01/23 12:18:39 tiggr Exp $  */

#define TLFDSTREAM_DECLARE_PRIVATE_METHODS
#define TLSOCKETSTREAM_DECLARE_PRIVATE_METHODS
#import "tl/support.h"
#import "tl/TLSocketStream.h"
#import "tl/TLString.h"
#import "tl/TLPatchedRoots.h"
#import <sys/socket.h>
#import <string.h>

@implementation TLSocketStream


+(TLSocketStream *) streamWithAddress: (id <TLNetworkAddress>) addr
{
  return ([[self gcAlloc] initWithAddress: addr]);
} /* +streamWithAddress: */

+(TLSocketStream *) streamWithFD: (int) d address: (id <TLNetworkAddress>) addr
{
  id r = [self streamWithFD: d];
  [r setAddress: (id) addr];
  return (r);
} /* +streamWithFD:address: */

-(TLSocketStream *) accept
{
  return (UNIMPLEMENTED_POINTER_METHOD);
} /* accept */

-(id <TLNetworkAddress>) address
{
  return (address);
} /* -address */

-(id) addressClass
{
  return (UNIMPLEMENTED_POINTER_METHOD);
} /* -addressClass */

-bind
{
  int r = bind (fd, [address address], [address addressSize]);
  if (r < 0)
    {
      [self warning: "bind: %s", ERRMSG];
      return (nil);
    }
  return (self);
} /* -bind */

-connect
{
  int r = connect (fd, [address address], [address addressSize]);
  if (r < 0)
    {
      [self warning: "connect: %s", ERRMSG];
      return (nil);
    }
  return (self);
} /* -connect */

-(id <TLString>) description
{
  return ([TLString stringWithFormat: @"<socket on %#>",
	   [(id) address description]]);
} /* -description */

-(void) gcReference
{
  MARK (address);
} /* -gcReference */

-initWithAddress: addr
{
  return (UNIMPLEMENTED_POINTER_METHOD);
} /* -initWithAddress: */

-listen
{
  int r = listen (fd, 255);
  if (r < 0)
    {
      [self warning: "listen: %s", ERRMSG];
      return (nil);
    }
  return (self);
} /* -listen: */

-(id <TLNetworkAddress>) peerAddress
{
  return (UNIMPLEMENTED_POINTER_METHOD);
} /* -peerAddress */

-(void) setAddress: (id <TLNetworkAddress>) addr
{
  address = addr;
} /* -setAddress: */

@end
