/* Implementation of TLUnixAddress class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLUnixAddress.m,v 1.3 1996/01/23 12:18:43 tiggr Exp $  */

#import "tl/TLUnixAddress.h"
#import "tl/TLUnixSocketStream.h"
#import "tl/TLPatchedRoots.h"
#import "tl/TLString.h"
#import <netdb.h>
#import <sys/socket.h>

@implementation TLUnixAddress

/******************** private methods ********************/

-initWithPathname: (id <TLString>) path
{
  const char *pn = [path cString];
  int l = [path length];

  pathname = path;

  if (l >= sizeof (address.sun_path))
    {
      [self warning: "Pathname too long `%s'", pn];
      INIT_RETURN_NIL ();
    }

  address.sun_family = AF_UNIX;
  memcpy (&address.sun_path, pn, l + 1);
  return (self);
} /* -initWithPathname: */

-initWithAddress: (void *) addr length: (int) len
{
  /* This is an insane sanity check, but it works.  */
  if (len != sizeof (address))
    {
      [self warning: "Bad address length %d != %d", len, sizeof (address)];
      INIT_RETURN_NIL ();
    }

  memcpy (&address, addr, len);
  pathname = [TLString stringWithCString: address.sun_path];
  return (self);
} /* -initWithAddress:length: */

/******************** public methods ********************/

+(TLUnixAddress *) addressWithPathname: (id <TLString>) pn
{
  return ([[self gcAlloc] initWithPathname: pn]);
} /* addressWithPathname: */

+(id <TLNetworkAddress>) addressWithAddress: (void *) addr length: (int) len
{
  return ([[self gcAlloc] initWithAddress: addr length: len]);
} /* +addressWithAddress:length: */

-(void *) address
{
  return (&address);
} /* -address */

-(int) addressSize
{
  return (sizeof (address));
} /* -addressSize */

-(id <TLString>) description
{
  return ([TLString stringWithFormat: @"<TLUnixAddress %p %@>",
	   self, pathname]);
} /* -description */

-(id <TLString>) pathname
{
  return (pathname);
} /* -pathname */

-(id) socketClass
{
  return ([TLUnixSocketStream self]);
} /* -socketClass */

/******************** garbage collection ********************/

-(void) gcReference
{
  MARK (pathname);
} /* -gcReference */

@end
