/* Implementation of TLUnixSocketStream class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLUnixSocketStream.m,v 1.3 1996/01/23 12:18:43 tiggr Exp $  */

#import "tl/support.h"
#import "tl/TLUnixSocketStream.h"
#import "tl/TLPatchedRoots.h"
#import "tl/TLString.h"
#import <sys/socket.h>
#import <sys/un.h>
#import <netdb.h>
#import <string.h>
#if HAVE_STRINGS_H
#import <strings.h>
#endif

@implementation TLUnixSocketStream

/******************** private methods ********************/

-(void) dealloc
{
  if (bind_p)
    {
      const char *pn = [[(id <TLUnixAddress>) address pathname] cString];

      if (unlink (pn))
	[self warning: "unlink %s: %s", pn, ERRMSG];
    }
  [super dealloc];
} /* -dealloc */

-initWithAddress: (id <TLUnixAddress>) addr
{
  address = addr;
  fd = socket (AF_UNIX, SOCK_STREAM, 0);
  if (fd == -1)
    {
      [self warning: "socket (%#): %s", address, ERRMSG];
      INIT_RETURN_NIL ();
    }
  return (self);
} /* -initWithAddress: */

/******************** public methods ********************/

-(TLSocketStream *) accept
{
  /* It's a pitty we need this struct here...  */
  struct sockaddr_un addr;
  int nfd, addr_len = sizeof (addr);
  id <TLNetworkAddress> naddr;

  bzero ((void *) &addr, sizeof (addr));
  nfd = accept (fd, (void *) &addr, &addr_len);
  if (nfd == -1)
    {
      [self warning: "accept: %s", ERRMSG];
      return (nil);
    }

  /* Sigh.  See remark in TLInetSocketStream.  */
  naddr = [(id) [self addressClass]
	   addressWithAddress: &addr length: sizeof (addr)];
  if (!naddr)
    return (nil);
  return ([[self class] streamWithFD: nfd address: naddr]);
} /* -accept */

-(id) addressClass
{
  return ([TLUnixAddress self]);
} /* -addressClass */

-bind
{
  id r = [super bind];
  if (!r)
    bind_p = 1;
  return (r);
} /* -bind */

@end
