/* Implementation of TLWrapStream class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLWrapStream.m,v 1.3 1996/01/23 12:18:44 tiggr Exp $  */

#define TLSTREAMSTREAM_DECLARE_PRIVATE_METHODS
#import "tl/TLWrapStream.h"
#import "tl/support.h"
#import <string.h>
#import <ctype.h>

@implementation TLWrapStream

-initWithStream: (id <TLOutputStream>) str
{
  if (![super initWithStream: (id) str])
    return (nil);

  tlbuffer_init (&buf);

  /* The default line length.  */
  line = 72;

  return (self);
} /* -initWithStream: */

-obeysNewlines
{
  return (obey_nl ? self : nil);
} /* -obeysNewlines */

-setObeysNewlines: nil_or_t
{
  obey_nl = !!nil_or_t;
  return (obey_nl ? nil_or_t : nil);
} /* -obeysNewlines */

-strictNewlines
{
  return (strict_nl ? self : nil);
} /* -strictNewlines */

-setStrictNewlines: nil_or_t
{
  strict_nl = !!nil_or_t;
  return (strict_nl ? nil_or_t : nil);
} /* -strictNewlines */

/******************** TLStream ********************/

-close
{
  int i, j;

  if (seen_nl)
    TLBUFFER_BUF (&buf)[TLBUFFER_NUM (&buf) - 1] = '\n';
  j = TLBUFFER_NUM (&buf);
  TLBUFFER_SET_NUM (&buf, 0);
  i = [stream writeBytes: j fromBuffer: TLBUFFER_BUF (&buf)];
  return (i == j ? self : nil);
} /* -close */

/******************** TLOutputStream ********************/

-flushOutput
{
  int i, j;

  if (!seen_nl)
    return (self);

  j = TLBUFFER_NUM (&buf);
  TLBUFFER_SET_NUM (&buf, 0);
  i = [stream writeBytes: j - 1 fromBuffer: TLBUFFER_BUF (&buf)];
  if (i != j - 1 || [stream writeByte: '\n'] == TL_EOF)
    return (nil);
  return ([super flushOutput]);
} /* -flushOutput */

-(int) writeByte: (char) c
{
  int r = [self writeBytes: 1 fromBuffer: &c];
  return (r == 1 ? c : TL_EOF);
} /* -writeByte: */

-(int) writeBytes: (int) n fromBuffer: (const char *) b
{
  int i, j, done = 0;
  char c;

  while (done != n)
    {
      tlbuffer_grow_if_needed (&buf, 1);
      TLBUFFER_BUF (&buf)[TLBUFFER_NUM (&buf)] = c = b[done++];
      TLBUFFER_SET_NUM (&buf, 1 + TLBUFFER_NUM (&buf));
      
      if (c == '\n')
	{
	  if (obey_nl || strict_nl || TLBUFFER_NUM (&buf) == 1)
	    {
	      j = TLBUFFER_NUM (&buf);
	      TLBUFFER_SET_NUM (&buf, 0);
	      i = [stream writeBytes: j fromBuffer: TLBUFFER_BUF (&buf)];
	      if (i != j)
		break;
	    }
	  else if (seen_nl)
	    {
	      seen_nl = 0;
	      j = TLBUFFER_NUM (&buf);
	      TLBUFFER_SET_NUM (&buf, 0);
	      i = [stream writeBytes: j - 2 fromBuffer: TLBUFFER_BUF (&buf)];
	      if (i != j - 2 || [stream writeBytes: 2 fromBuffer: "\n\n"] != 2)
		break;
	    }
	  else
	    {
	      TLBUFFER_BUF (&buf)[TLBUFFER_NUM (&buf) - 1] = ' ';
	      seen_nl = 1;
	    }
	}
      else
	{
	  seen_nl = 0;
	  if (TLBUFFER_NUM (&buf) >= line && !isspace (c) && !strict_nl)
	    {
	      for (i = TLBUFFER_NUM (&buf) - 1; i >= 0; i--)
		if (isspace (TLBUFFER_BUF (&buf)[i]))
		  break;
	      if (i > 0)
		{
		  for (j = i - 1; j >= 0; j--)
		    if (!isspace (j))
		      break;

		  /* I points to the last space, J to just before the first.  */
		  if (j < 0)
		    {
		      /* What to do with space-only lines which need to be
			 wrapped?  */
		    }
		  else
		    {
		      int k = [stream writeBytes: j + 1
			       fromBuffer: TLBUFFER_BUF (&buf)];
		      if (k != j + 1)
			break;
		    }
		  if ([stream writeByte: '\n'] == TL_EOF)
		    break;
		  TLBUFFER_SET_NUM (&buf, TLBUFFER_NUM (&buf) - (i + 1));
		  memmove (TLBUFFER_BUF (&buf), TLBUFFER_BUF (&buf) + i + 1,
			   TLBUFFER_NUM (&buf));
		}
	    }
	}
    }
  return (done);
} /* -writeBytes:fromBuffer: */

/******************** garbage collection ********************/

-(void) dealloc
{
  tlbuffer_dealloc (&buf);
} /* -dealloc */

@end
