/* Implementation of simple, fast hashtable.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: rchash.m,v 1.3 1996/01/23 12:18:45 tiggr Exp $  */

#include "tl/support.h"
#include "tl/rchash.h"
#include <stdlib.h>

#define HASH(ADDR, CAP) (((int) (ADDR)) % (CAP))

static void
rch_grow (rch_table *t)
{
  rch_bucket **nt;
  rch_bucket *ob, *nb;
  int oi, oj, ni, nc;
#if 0
  int max = 0;
  int sizes[1000];

  bzero (sizes, sizeof (sizes));
#endif

  nc = t->cap * 2 + 1;
  nt = xcalloc (nc, sizeof (*nt));

  for (oi = 0; oi < t->cap; oi++)
    {
      for (oj = 0, ob = t->table[oi]; ob; oj++, ob = nb)
	{
	  nb = ob->next;
	  ni = HASH (ob->key, nc);
	  ob->next = nt[ni];
	  nt[ni] = ob;
	}
#if 0
      if (oj > max)
	max = oj;
      if (oj > 1000)
	abort ();
      sizes[oj]++;
#endif
    }

#if 0
  {
    int i;
    for (i = 0; i <= max; i++)
      fprintf (stderr, "%3d %d\n", i, sizes[i]);
    fprintf (stderr, "rch_grow %d->%d max=%d\n", t->cap, nc, max);
  }
#endif

  xfree (t->table);
  t->table = nt;
  t->cap = nc;
} /* rch_grow */

rch_table *
rch_create (int cap)
{
  rch_table *t = xmalloc (sizeof (*t));

  t->cap = cap;
  t->num = 0;
  t->table = xcalloc (t->cap, sizeof (*t->table));
  return (t);
} /* rch_create */

rch_bucket *
rch_bucket_for_key (rch_table *t, void *key, int create_p)
{
  int index = HASH (key, t->cap);
  rch_bucket **b, *n;

  for (b = &t->table[index]; *b && (*b)->key != key; b = &(*b)->next);
  if (*b || !create_p)
    return (*b);

  n = xmalloc (sizeof (*n));

  if (t->num++ >= t->cap)
    {
      rch_grow (t);
      index = HASH (key, t->cap);
    }

  n->key = key;
  n->next = t->table[index];
  t->table[index] = n;
  n->value.i = 0;

  return (n);
} /* rch_bucket_for_key */

void
rch_remove_key (rch_table *t, void *key)
{
  int index = HASH (key, t->cap);
  rch_bucket **b, *n;

  for (b = &t->table[index]; *b && (*b)->key != key; b = &(*b)->next);
  if (*b)
    {
      n = *b;
      *b = n->next;
      xfree (n);
      t->num--;
    }
} /* rch_remove_key */
