/* Allocation tests.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: allocation.m,v 1.2 1996/01/23 12:19:04 tiggr Exp $  */

#import <stdarg.h>
#import <stdio.h>
#import <signal.h>
#import <tl/tl.h>

NSAutoreleasePool *pool;
id o1, o2;

void
fatal (const char *fmt, ...)
{
  va_list ap;
  va_start (ap, fmt);
  vfprintf (stderr, fmt, ap);
  fputc ('\n', stderr);
  va_end (ap);
  exit (2);
} /* fatal */

void
c1 (int sig)
{
  fatal ("c1");
} /* c1 */

int
main (int argc, char **argv)
{
  tl_init ();

  signal (SIGIOT, c1);
  o1 = [[Object alloc] init];
  if ([o1 retainCount] != 0)
    fatal ("fail:%d: alloc is not retained", __LINE__);
  [o1 retain];
  if ([o1 retainCount] != 1)
    fatal ("fail:%d: retain does not retain", __LINE__);
  else
    {
      [o1 release];
      if ([o1 retainCount] != 0)
	fatal ("fail:%d: release does not release", __LINE__);
    }
  [o1 release];

  /* XXX How to check if it is really freed?  */

  return (0);
} /* main */
