/* Test standalone lisp evaluator.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: teval.m,v 1.8 1996/10/01 10:26:57 tiggr Exp $  */

#import <tl/tl.h>
#import <signal.h>
#import <string.h>

/* Catch SIGINT.  */
void
catch_int (int sig)
{
  extern int tl_quit_inhibit, tl_quit_request;

  /* Re-install the signal handler.  */
  signal (SIGINT, catch_int);

  if (tl_quit_inhibit)
    tl_quit_request++;
  else
    [TLObject error: "quit"];
} /* catch_int */

int
main (int argc, char **argv)
{
  TLCons *arglist = nil;
  int daemon_p = 0, interactive = 1;

  tl_init ();

  signal (SIGINT, catch_int);

  {
    TLCons *last = nil;
    int i;

    for (i = 0; i < argc; i++)
      if (!strcmp (argv[i], "--teval-daemon"))
	daemon_p = 1;
      else
	{
	  TLCons *s;

	  s = CONS ([TLString stringWithCString: argv[i]], nil);
	  if (last)
	    [last rplacd: s], last = s;
	  else
	    last = arglist = s;
	}
    [TLSymbol symbolWithName: @"argv" varValue: arglist];

    if (argc > 1)
      interactive = 0;

    [TLSymbol symbolWithName: @"user-interactive"
     varValue: (interactive ? Qt : nil)];
  }

  if (daemon_p)
    {
      char *s = strrchr (argv[0], '/');
      id sls;

      s = s ? s + 1 : argv[0];
      sls = Fstream_on_syslog ([TLString stringWithCString: s]);
      V_stdout_ = V_stderr_ = sls;
      V_stdin_ = nil;
      formac (sls, @"teval daemon startup\n");
    }

  Feval_string (
@"\n\
;;; Default load path is just the current directory.\n\
(setq load-path\n\
      (mapcar: (componentsSeparatedByString:\n\
		(or (getenv \"TL_LOAD_PATH\") \".:"
			PATH_PREFIX @"/lib/tl/tlisp\")\n\
		\":\") 'self))\n\
\n\
;;; Simple load.  Overridden later on.\n\
(defun load (filename)\n\
  (condition-case reason\n\
      (let* (line (file [TLFILEStream streamWithFileNamed: filename\n\
				      alongPath: load-path])\n\
	     (lexer [TLLLex lexerWithStream: file]))\n\
	(unwind-protect (while (not (eq (intern \"*lex-eof*\")\n\
				        (setq line [lexer read])))\n\
			   [line eval])\n\
	  [file close]))\n\
    (t (formac t \"caught %#\n\" reason))))\n\
\n\
;;; Load the file `~/loadup.tl'.  That should be a link to\n\
;;; .../tl/tlisp/loadup.tl\n\
(let ((init-file \"loadup.tl\"))\n\
  (condition-case reason\n\
   (load init-file)\n\
   (t (formac t \"error while reading %@: %@\" init-file reason))))\n\
\n\
(load \"teval.tl\")\n\
");

  return (0);
} /* main */
