/* Interface to the TLMimeItem class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLMimeItem.h,v 1.3 1996/01/23 12:19:20 tiggr Exp $  */

#ifdef TLMIMEITEM_DECLARE_PRIVATE_METHODS
#define TLOBJECT_DECLARE_PRIVATE_METHODS
#endif

#import "TLString.h"
#import "TLVector.h"

@interface TLMimeItem: TLObject
{
  /* The headers, as a cons cell of (header . value).  Continuation lines
     are appended.  */
  TLVector *headers;

  /* The content-type/content-format (duplicated from the information in the
     HEADERS).  */
  id <TLString> content_type, content_format, content_options;

  /* The length of the contents.  */
  id <TLNumber> content_length;

  /* Content transfer encoding value.  */
  id <TLString> xfer_encoding;

  /* The MIME version.  */
  id <TLString> mime_version;

  /* A string of the contents, or a vector of the mime items.  */
  id contents;
}

+(TLMimeItem *) mimeItemWithHeaders: (id <TLString>) h range: (id <TLRange>) r;
-initWithHeaders: (id <TLString>) h range: (id <TLRange>) r;

/* Retrieving information.  */
-(TLNumber *) contentLength;
-(TLString *) contentType;
-(TLString *) contentFormat;
-(TLString *) contentTypeOptions;
-(TLString *) contentTransferEncoding;
-(id <TLVector>) headers;
-(TLString *) mimeVersion;
-(id <TLString>) header: (id <TLString>) h;

/* Manipualting the contents.  */
-readContents: (id <TLInputStream>) stream;
-setContents: (id <TLString>) contents;
-contents;

#ifdef TLMIMEITEM_DECLARE_PRIVATE_METHODS

/* Add the indicated HEADER and VALUE to the HEADERS, possibly digesting the
   information for cahcing.  */
-(void) addHeader: (TLString *) header value: (TLString *) value;

#endif

@end
