/* Interface to the TLURL class.
   This file is part of TL, Tiggr's Library.
   Written by Tiggr <tiggr@es.ele.tue.nl>
   Copyright (C) 1995, 1996 Pieter J. Schoenmakers
   TL is distributed WITHOUT ANY WARRANTY.
   See the file LICENSE in the TL distribution for details.

   $Id: TLURL.h,v 1.6 1996/01/23 12:19:26 tiggr Exp $  */

#import "TLString.h"
#import "TLVector.h"

@protocol TLURL

-(id <TLString>) scheme;
-(id <TLString>) hostname;

/* Return the port to contact.  This is either the explicit port in the URL
   or the port derived from the protocol.  */
-(id <TLNumber>) port;

-(id <TLVector>) path;
-(id <TLVector>) parameters;
-(id <TLVector>) arguments;
-(id <TLString>) fragment;

-(BOOL) explicitPortP;

@end

@interface TLURL: TLObject <TLURL>
{
  /* The scheme and hostname.  */
  id <TLString> scheme, hostname;

  /* `/' path elements, XXX _unquoted_.  */
  id <TLMutableVector> path;

  /* The PATH as a filename string.  */
  id <TLString> filename;

  /* `;' parameters, _unquoted_.  */
  id <TLMutableVector> parms;

  /* `?' arguments, _unquoted_.  */
  id <TLMutableVector> args;

  /* `#' fragment, _unquoted_.  */
  id <TLString> frag;

  /* The port number.  */
  int port;
}

+(id <TLURL>) urlWithString: (id <TLString>) name;

+(id <TLURL>) urlWithString: (id <TLString>) name
		 relativeTo: (id <TLURL>) url;

+(id <TLURL>) urlWithURL: (id <TLURL>) url
	      relativeTo: (id <TLURL>) base;

-initWithString: (id <TLString>) in_name
     relativeTo: (id <TLURL>) base;

-initWithURL: (id <TLURL>) url
  relativeTo: (id <TLURL>) base;

/* Modifying.  */
-(void) addPathElement: (id) e;

/* Get the last item from the PATH.  */
-(id <TLString>) basename;

/* Retrieving information.  */
-(id <TLString>) filename;

#ifdef TLURL_DECLARE_PRIVATE_METHODS

-(TLVector *) gatherFrom: (char) sc intermediate: (char) ic
  from: (const char *) s with: (int *) bp length: (int) l;

#endif

@end
